/* verify_sender/mcom.c - some common code for two main(), daemon and tester */

/*
 * Copyright (c) 2003 Valentin Nechayev <netch@netch.kiev.ua> All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met: 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 2.
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * $Id: mcom.c,v 1.2 2003/07/31 14:35:20 netch Exp $
 */

#include <err.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "mcom.h"
#include "verify.h"
#ifndef HAVE_ERRC
#include <stdarg.h>
#endif

char *
xstrdup(const char *s)
{
    char *t;

    t = xcalloc(1, 1 + strlen(s));
    strcpy(t, s);
    return t;
}

void *
xcalloc(size_t n, size_t s)
{
    void *p = calloc(n, s);

    if (!p)
	errx(1, "xcalloc(): no memory");
    return p;
}

char *
xstrndup(const char *s, size_t nn)
{
    char *t;
    size_t n = strnlen(s, nn);

    t = malloc(n + 1);
    if (!t)
	errx(1, "xstrndup(): no memory");
    memmove(t, s, n);
    t[n] = 0;
    return t;
}

size_t
strnlen(const char *src, size_t maxlen)
{
    void *p;

    p = memchr(src, 0, maxlen);
    if (p)
	return (const char *)p - src;
    return maxlen;
}

#ifdef USE_OWN_STRLCPY
size_t 
strlcpy(char *dst, const char *src, size_t siz)
{
    register char *d = dst;
    register const char *s = src;
    register size_t n = siz;

    /* Copy as many bytes as will fit */

    if (n != 0 && --n != 0) {
	do {
	    if ((*d++ = *s++) == 0)
		break;
	} while (--n != 0);
    }

    /* Not enough room in dst, add NUL and traverse rest of src */

    if (n == 0) {
	if (siz != 0)
	    *d = '\0';		/* NUL-terminate dst */
	while (*s++);
    }
    return (s - src - 1);	/* count does not include NUL */
}
#endif

#ifndef HAVE_ERRC
void
errc(int es, int err, const char *format,...)
{
    va_list va;

    va_start(va, format);
    errno = err;
    verr(es, format, va);
    va_end(va);
}
#endif
